import unittest

import yaml
from pandas import DataFrame
from yaml import SafeLoader

from caddo_file_parser.caddo_file_parser import CaddoFileParser
from caddo_file_parser.models.caddo_file import CaddoFile
from caddo_file_parser.models.index_set import IndexSet
from caddo_file_parser.models.run import Run
from caddo_file_parser.settings.generation_settings import GenerationSettings
from caddo_file_parser.settings.generation_settings_loader import GenerationSettingsLoader


class FileSavingTest(unittest.TestCase):
    def test_save_caddo_file(self):
        caddo_file_parser = CaddoFileParser()
        caddo_file = self._create_caddo_file()
        caddo_file_parser.create_file(caddo_file)

    def _create_caddo_file(self):
        settings_loader = GenerationSettingsLoader()
        with open("settings-data.yaml", 'r') as file:
            settings = settings_loader.load_settings_object(yaml.load(file, Loader=SafeLoader))
            caddo_file = CaddoFile(
                runs=[
                    Run(
                        number=1,
                        index_sets=[
                            IndexSet(1, [1, 2, 0], [2, 0, 1], 10),
                            IndexSet(0, [2, 2, 2], [2, 0, 0], 10)
                        ],
                        seed=10
                    ),
                    Run(
                        number=0,
                        index_sets=[
                            IndexSet(1, [1, 1, 2], [1, 2, 1], 11),
                            IndexSet(0, [0, 1, 2], [1, 0, 0], 11)
                        ],
                        seed=11
                    ),
                ],
                data=DataFrame(
                    data=[[0, 1, 2], [1, 3, 4], [2, 5, 6]],
                    columns=["idx", "x__col_A", "y__col_B"]
                ),
                settings=settings,
                seeds=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
            )
            return caddo_file



class GenerationSettings:
    data_settings_file_path = ''
    data_input_path = ''
    data_input_separator = ''
    data_extraction_function_path = ''
    data_splitting_folding_number = 0
    data_splitting_runs = 0
    data_output_file_name = ''
    data_splitting_folding_method = ''
    data_splitting_folding_seeds_from_list = []
    data_splitting_folding_seeds_file_path = ''
    data_output_file_separator = ''

# SAMPLE FILE
# data:
#   input:
#     path: data.csv
#     separator: $
#   extraction:
#     function:
#       path: extraction_function
#   output:
#     file:
#       name: results.csv
#       separator: $
#   splitting:
#     folding:
#       method: KFOLDS
#       number: 10
#       runs: 10
#       seeds:
#         from_list: [1,2,3,4,5, 6, 7, 8, 9, 10] //only one of them should be filled, the second one live empty
#         from_path: seeds.yaml




from caddo_file_parser.models.index_set import IndexSet


class Run:
    def __init__(self, index_sets: [IndexSet], seed: int, number: int):
        self.index_sets = index_sets
        self.seed = seed
        self.number = number



class IndexSet:
    def __init__(self, number: int, train_indexes: [int], test_indexes: [int], seed: int):
        self.number = number
        self.train_indexes = train_indexes
        self.test_indexes = test_indexes
        self.seed = seed

    @staticmethod
    def of(index_set_yaml_file):
        number = index_set_yaml_file["number"]
        train_indexes = index_set_yaml_file["train_indexes"]
        test_indexes = index_set_yaml_file["test_indexes"]
        seed = index_set_yaml_file["seed"]
        index_set = IndexSet(number, train_indexes, test_indexes, seed)
        return index_set
